@@ -51,7 +51,21 @@ function firewall_rule_item_proto($filterent)
                 break;
         }
     } else {
-        $record_ipprotocol = "IPv4 ";
+        // when ipprotocol is not set, pf would normally figure out the ip proto itself.
+        // reconstruct ipproto depending on source/destination address.
+        if (!empty($filterent['from']) && is_ipaddr(explode("/", $filterent['from'])[0])) {
+            $record_ipprotocol = strpos($filterent['from'], ":") === false ? "IPv4 " :  "IPv6 ";
+        } elseif (!empty($filterent['to']) && is_ipaddr(explode("/", $filterent['to'])[0])) {
+            $record_ipprotocol = strpos($filterent['to'], ":") === false ? "IPv4 " :  "IPv6 ";
+        } elseif (isset($filterent['source']['address'])
+                    && is_ipaddr(explode("/", $filterent['source']['address'])[0])) {
+            $record_ipprotocol = strpos($filterent['source']['address'], ":") === false ? "IPv4 " : "IPv6 ";
+        } elseif (isset($filterent['destination']['address'])
+                    && is_ipaddr(explode("/", $filterent['destination']['address'])[0])) {
+            $record_ipprotocol = strpos($filterent['destination']['address'], ":") === false ? "IPv4 " : "IPv6 ";
+        } else {
+            $record_ipprotocol = "IPv4+6 ";
+        }
     }
     $icmptypes = array(
       "" => gettext("any"),
@@ -126,7 +140,7 @@ function firewall_rule_item_proto($filterent)
 function firewall_rule_item_icons($filterent)
 {
     $result = "";
-    if (!empty($filterent['direction']) && $filterent['direction'] == "in") {
+    if (empty($filterent['direction']) || $filterent['direction'] == "in") {
         $result .= sprintf(
             "<i class=\"fa fa-long-arrow-right fa-fw text-info\" data-toggle=\"tooltip\" title=\"%s\"></i>",
             gettext("in")
@@ -136,25 +150,30 @@ function firewall_rule_item_icons($filterent)
             "<i class=\"fa fa-long-arrow-left fa-fw\" data-toggle=\"tooltip\" title=\"%s\"></i>",
             gettext("out")
         );
+    } else {
+        $result .= sprintf(
+            "<i class=\"fa fa-exchange fa-fw\" data-toggle=\"tooltip\" title=\"%s\"></i>",
+            gettext("any")
+        );
     }
-    if (!empty($filterent['floating'])) {
-        if (isset($filterent['quick']) && $filterent['quick'] === 'yes') {
-            $result .= sprintf(
-                "<i class=\"fa fa-flash fa-fw text-warning\" data-toggle=\"tooltip\" title=\"%s\"></i>",
-                gettext('first match')
-            );
-        } else {
-          $result .= sprintf(
-              "<i class=\"fa fa-flash fa-fw text-muted\" data-toggle=\"tooltip\" title=\"%s\"></i>",
-              gettext('last match')
-          );
-        }
+    if (empty($filterent['floating']) && $filterent['quick'] === null){
+        $is_quick = true;
+    } elseif (!empty($filterent['floating']) && $filterent['quick'] === null) {
+        $is_quick = false;
+    } else {
+        $is_quick = $filterent['quick'];
     }
-    if (isset($filterent['log'])) {
-          $result .= sprintf(
-              "<i class=\"fa fa-info-circle fa-fw %s\"></i>",
-              !empty($filterent['disabled']) ? 'text-muted' : 'text-info'
-          );
+
+    if ($is_quick) {
+        $result .= sprintf(
+            "<i class=\"fa fa-flash fa-fw text-warning\" data-toggle=\"tooltip\" title=\"%s\"></i>",
+            gettext('first match')
+        );
+    } else {
+      $result .= sprintf(
+          "<i class=\"fa fa-flash fa-fw text-muted\" data-toggle=\"tooltip\" title=\"%s\"></i>",
+          gettext('last match')
+      );
     }
 
     return $result;
@@ -176,6 +195,15 @@ function firewall_rule_item_action($filterent)
         return "fa fa-play fa-fw text-muted";
     }
 }
+
+function firewall_rule_item_log($filterent)
+{
+    if ($filterent['log'] == true) {
+        return "fa fa-info-circle fa-fw text-info";
+    } else {
+        return "fa fa-info-circle fa-fw text-muted";
+    }
+}
 /***********************************************************************************************************
  *
  ***********************************************************************************************************/
@@ -193,11 +221,11 @@ if ($_SERVER['REQUEST_METHOD'] === 'POST') {
         // id found and valid
         $id = $pconfig['id'];
     }
-    if (isset($pconfig['apply'])) {
+    if (isset($pconfig['act']) && $pconfig['act'] == "apply") {
         system_cron_configure();
         filter_configure();
         clear_subsystem_dirty('filter');
-        $savemsg = gettext('The settings have been applied and the rules are now reloading in the background.');
+        $savemsg = get_std_save_message();
     } elseif (isset($pconfig['act']) && $pconfig['act'] == 'del' && isset($id)) {
         // delete single item
         if (!empty($a_filter[$id]['associated-rule-id'])) {
@@ -262,7 +290,25 @@ if ($_SERVER['REQUEST_METHOD'] === 'POST') {
         }
         write_config();
         mark_subsystem_dirty('filter');
-        header(url_safe('Location: /firewall_rules.php?if=%s', array($current_if)));
+        $response = array("id" => $id);
+        $response["new_label"] = !isset($a_filter[$id]['disabled']) ?  gettext("Disable Rule") : gettext("Enable Rule");
+        $response["new_state"] = !isset($a_filter[$id]['disabled']) ;
+        echo json_encode($response);
+        exit;
+    } elseif (isset($pconfig['act']) && $pconfig['act'] == 'log' && isset($id)) {
+        // toggle logging
+        if(isset($a_filter[$id]['log'])) {
+            unset($a_filter[$id]['log']);
+        } else {
+            $a_filter[$id]['log'] = true;
+        }
+        write_config();
+        mark_subsystem_dirty('filter');
+        //header(url_safe('Location: /firewall_rules.php?if=%s', array($current_if)));
+        $response = array("id" => $id);
+        $response["new_label"] = isset($a_filter[$id]['log']) ?  gettext("Disable Log") : gettext("Enable Log");
+        $response["new_state"] = isset($a_filter[$id]['log']) ;
+        echo json_encode($response);
         exit;
     }
 }
@@ -385,13 +431,59 @@ $( document ).ready(function() {
     $("#iform").submit();
   });
 
+  // link move buttons
+  $("#btn_apply").click(function(event){
+    event.preventDefault();
+    $("#action").val("apply");
+    $("#iform").submit();
+  });
+
   // link toggle buttons
   $(".act_toggle").click(function(event){
-    event.preventDefault();
-    var id = $(this).attr("id").split('_').pop(-1);
-    $("#id").val(id);
-    $("#action").val("toggle");
-    $("#iform").submit();
+      event.preventDefault();
+      let target = $(this);
+      let id = target.attr("id").split('_').pop(-1);
+      $.ajax("firewall_rules.php",{
+          type: 'post',
+          cache: false,
+          dataType: "json",
+          data: {'act': 'toggle', 'id': id},
+          success: function(response) {
+              target.prop('title', response['new_label']).tooltip('fixTitle').tooltip('hide');
+              if (response['new_state']) {
+                  target.find('span').removeClass('text-muted').addClass('text-success');
+              } else {
+                  target.find('span').removeClass('text-success').addClass('text-muted');
+              }
+              $("#fw-alert-box").removeClass("hidden");
+              $(".fw-alert-messages").addClass("hidden");
+              $("#fw-alert-changes").removeClass("hidden");
+          }
+      });
+  });
+
+   // link log buttons
+  $(".act_log").click(function(event){
+      event.preventDefault();
+      let target = $(this);
+      let id = target.attr("id").split('_').pop(-1);
+      $.ajax("firewall_rules.php",{
+          type: 'post',
+          cache: false,
+          dataType: "json",
+          data: {'act': 'log', 'id': id},
+          success: function(response) {
+              target.prop('title', response['new_label']).tooltip('fixTitle').tooltip('hide');
+              if (response['new_state']) {
+                  target.find('i').removeClass('text-muted').addClass('text-info');
+              } else {
+                  target.find('i').removeClass('text-info').addClass('text-muted');
+              }
+              $("#fw-alert-box").removeClass("hidden");
+              $(".fw-alert-messages").addClass("hidden");
+              $("#fw-alert-changes").removeClass("hidden");
+          }
+      });
   });
 
   // watch scroll position and set to last known on page load
@@ -542,10 +634,19 @@ $( document ).ready(function() {
     <div class="container-fluid">
       <div class="row">
         <?php print_service_banner('firewall'); ?>
-        <?php if (isset($savemsg)) print_info_box($savemsg); ?>
-        <?php if (is_subsystem_dirty('filter')): ?><p>
-        <?php print_info_box_apply(gettext("The firewall rule configuration has been changed.<br />You must apply the changes in order for them to take effect."));?>
-        <?php endif; ?>
+        <div id="fw-alert-box" class="col-xs-12 <?=!is_subsystem_dirty('filter') && !isset($savemsg) ? "hidden":"";?>">
+          <div class="alert alert-info" role="alert">
+            <div id="fw-alert-changes" class="fw-alert-messages <?=!is_subsystem_dirty('filter') ? "hidden":"";?>">
+                <label for="btn_apply">
+                  <?=gettext("The firewall rule configuration has been changed.<br />You must apply the changes in order for them to take effect.");?>
+                </label>
+                <button id="btn_apply" class="btn btn-primary pull-right" value="Apply changes"><?=gettext("Apply changes");?></button>
+            </div>
+            <div id="fw-alert-message" class="fw-alert-messages <?=!isset($savemsg) ? "hidden":"";?>">
+                <?=isset($savemsg) ? $savemsg : "";?>
+            </div>
+          </div>
+        </div>
 <?php
           $interface_has_rules = false;
           foreach ($a_filter as $i => $filterent) {
@@ -618,13 +719,20 @@ $( document ).ready(function() {
                 filter_core_bootstrap($fw);
                 plugins_firewall($fw);
                 foreach ($fw->iterateFilterRules() as $rule):
-                    if ($rule->getInterface() == $selected_if && $rule->isEnabled()):
+                    $is_selected = $rule->getInterface() == $selected_if || (
+                        $rule->getInterface() == "" && $selected_if == "FloatingRules"
+                    );
+                    if ($rule->isEnabled() && $is_selected):
                         $filterent = $rule->getRawRule();
+                        $filterent['quick'] = !isset($filterent['quick']) || $filterent['quick'];
+                        legacy_html_escape_form_data($filterent);
                         $rule_stats = !empty($rule->getLabel()) ? $all_rule_stats[$rule->getLabel()] : array();?>
                     <tr class="internal-rule" style="display: none;">
                       <td><i class="fa fa-magic"></i></td>
                       <td>
-                          <span class="<?=firewall_rule_item_action($filterent);?>"></span><?=firewall_rule_item_icons($filterent);?>
+                          <span class="<?=firewall_rule_item_action($filterent);?>"></span>
+                          <?=firewall_rule_item_icons($filterent);?>
+                          <i class="<?=firewall_rule_item_log($filterent);?>"></i>
                       </td>
                       <td class="view-info">
                           <?=firewall_rule_item_proto($filterent);?>
@@ -677,10 +785,13 @@ $( document ).ready(function() {
                       <input class="rule_select" type="checkbox" name="rule[]" value="<?=$i;?>"  />
                     </td>
                     <td>
-                      <a href="#" class="act_toggle" id="toggle_<?=$i;?>" data-toggle="tooltip" title="<?=(empty($filterent['disabled'])) ? gettext("Disable") : gettext("Enable");?>">
+                      <a href="#" class="act_toggle" id="toggle_<?=$i;?>" data-toggle="tooltip" title="<?=(empty($filterent['disabled'])) ? gettext("Disable Rule") : gettext("Enable Rule");?>">
                         <span class="<?=firewall_rule_item_action($filterent);?>"></span>
                       </a>
                       <?=firewall_rule_item_icons($filterent);?>
+                      <a href="#" class="act_log" id="toggle_<?=$i;?>" data-toggle="tooltip" title="<?=(empty($filterent['log'])) ? gettext("Enable Log") : gettext("Disable Log");?>">
+                        <i class="<?=firewall_rule_item_log($filterent);?>"></i>
+                      </a>
                     </td>
                     <td class="view-info">
                         <?=firewall_rule_item_proto($filterent);?>
@@ -867,10 +978,8 @@ $( document ).ready(function() {
                           <td style="width:100px"><?=gettext("log");?></td>
                           <td style="width:16px"><span class="fa fa-long-arrow-right text-info"></span></td>
                           <td style="width:100px"><?=gettext("in");?></td>
-<?php if ($selected_if == 'FloatingRules'): ?>
                           <td style="width:16px"><span class="fa fa-flash text-warning"></span></td>
                           <td style="width:100px"><?=gettext("first match");?></td>
-<?php endif ?>
                         </tr>
                         <tr>
                           <td><span class="fa fa-play text-muted"></span></td>
@@ -886,10 +995,8 @@ $( document ).ready(function() {
                           <td class="nowrap"><?=gettext("log (disabled)");?></td>
                           <td style="width:16px"><span class="fa fa-long-arrow-left"></span></td>
                           <td style="width:100px"><?=gettext("out");?></td>
-<?php if ($selected_if == 'FloatingRules'): ?>
                           <td style="width:16px"><span class="fa fa-flash text-muted"></span></td>
                           <td style="width:100px"><?=gettext("last match");?></td>
-<?php endif ?>
                         </tr>
                       </table>
                     </td>
@@ -910,7 +1017,7 @@ $( document ).ready(function() {
                   <tr class="hidden-xs hidden-sm">
                     <td>
 <?php if ('FloatingRules' != $selected_if): ?>
-                      <?= sprintf(gettext('%s rules are evaluated on a first-match basis (i.e. ' .
+                      <?= sprintf(gettext('%s rules are evaluated on a first-match basis by default (i.e. ' .
                         'the action of the first rule to match a packet will be executed). ' .
                         'This means that if you use block rules, you will have to pay attention ' .
                         'to the rule order. Everything that is not explicitly passed is blocked ' .
